<?php
/*
 * Allow players to add tracks to the 'jukebox' so they can play favorites without waiting
 * Each player can only have one track in jukebox at a time
 */

include_once('includes/rasp.funcs.php');

//Register events and chat commands with aseco
Aseco::registerEvent('onEndRace', 'rasp_endrace');
Aseco::addChatCommand('jukebox', 'Sets a track to be played next.');
Aseco::addChatCommand('list', 'Lists tracks currently on the server');
Aseco::registerEvent('onNewChallenge', 'rasp_newtrack');
Aseco::addChatCommand('add', 'Adds a track to the server, directly from TMX');
Aseco::addChatCommand('y', 'Votes for a TMX map');

function rasp_newtrack(&$aseco, $data)
	{
	global $buffersize, $jb_buffer, $tmxplaying, $tmxvotes, $tmxplayed, $tmxadd, $jukebox;

	$tmxvotes = array();
	$tmxadd = array();

	if (sizeof($jb_buffer) > $buffersize)		// drop the first track from stack
		{
		$drop_value = array_shift($jb_buffer);
		}
	$jb_buffer[] = $data->uid;					// add current track to the stack

	$current = array_shift($jukebox);
	if ( $current['uid'] != $data->uid && isset($current['uid']) )		// track playing isn't next from jukebox, put it back so it's still next in jukebox
		{
		array_unshift($jukebox, $current);
		return;
		}

	if ($tmxplaying != false)
		{
		$tmxplayed = $tmxplaying;
		$tmxplaying = false;
		}
	else
		{
		if ($tmxplayed != false)
			{
			$set = $aseco->client->addCall('RemoveChallenge', array($tmxplayed));
			if (!$aseco->client->multiquery())
				{
				trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage());
				$response = array();
				}
			else
				{
				$tmxplayed = false;
				}
			}
		}
	}

function rasp_endrace(&$aseco, $challenge)
	{
	global $jukebox, $aseco, $tmxplaying;

	if (sizeof($jukebox) > 0)
		{
		$next = array_shift($jukebox);
		array_unshift($jukebox, $next);
		if ($next['tmx'])
			{
			if ( $aseco->debug )
				{
				$aseco->console_text('{RASP} TMX challenge filename is ' . $next['FileName']);
				}

			$aseco->client->addCall('AddChallenge', array($next['FileName']));
			if (!$aseco->client->multiquery())
				{
				trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage());
				return;
				}
			}
		$aseco->client->addCall('ChooseNextChallenge', array($next['FileName']));
		if (!$aseco->client->multiquery())
			{
			trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage());
			}
		else
			{
			$msg = '{RASP Jukebox} Setting Next Challenge to ' . $next['Name'];
			if ( $next['tmx'] )
				{
				$msg = '{RASP Jukebox} Setting Next Challenge to ' . $next['Name'] . ', file downloaded from TMX';
				}
			$aseco->console_text($msg);
			$response = $aseco->client->getResponse();
			$message = '{#server}>{#emotic} The next track will be {#highlite}' . $next['Name'] . '{#emotic} as requested by {#highlite}' . stripFormatting($next['Nick']);
			$message = $aseco->formatColors($message);
			$aseco->addCall('ChatSendServerMessage', array($message));
			if($next['tmx'])
				{
				$tmxplaying = $next['FileName'];
				}
			}
		}
	}


function chat_jukebox(&$aseco, &$command)
{
	global $player, $jukebox, $aseco, $rasp, $jb_buffer;
	$player = $command['author'];
	$login = $player->login;
	if (is_numeric($command['params']) && $command['params'] >= 0)
	{
		if(count($player->tracklist) == 0)
		{
			$message = $aseco->formatColors('{#server}>{#emotic} Use \'/list\' first (use /list help for more info)');
			$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
			return;
		}
		$jid = ltrim($command['params'], '0');
		$jid--;
		foreach ($jukebox as $key)
		{
			if ($player->nickname == $key['Nick'])
			{
				$message = $aseco->formatColors('{#server}>{#emotic} You already have a track in the jukebox! Wait till it\'s been played before adding another.');
				$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
				return;
			}
		}
		if (array_key_exists($jid, $player->tracklist)) 		// find track by given #
		{
			$uid = $player->tracklist[$jid]['uid'];
			if (array_key_exists($uid, $jukebox)) 				// find by uid in jukebox
			{
				$message = $aseco->formatColors('{#server}>{#emotic} This track has already been added to the jukebox, pick another one');
				$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
				return;
			}
			elseif (in_array($uid, $jb_buffer) && !$aseco->isAdmin($login))
			{
				$message = $aseco->formatColors('{#server}>{#emotic} This track has been played recently. Wait a while before adding it again.');
				$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
				return;
			}
			$jukebox[$uid]['FileName'] = $player->tracklist[$jid]['filename'];
			$jukebox[$uid]['Name'] = $player->tracklist[$jid]['name'];
			$jukebox[$uid]['Nick'] = $player->nickname;
			$jukebox[$uid]['tmx'] = false;
			$jukebox[$uid]['uid'] = $uid;
			$jukebox[$uid]['env'] = $player->tracklist[$jid]['env'];
			$message = formatText($rasp->messages['JUKEBOX'][0], $player->tracklist[$jid]['name'], stripFormatting($player->nickname));
			$message = $aseco->formatColors($message);
			$aseco->addCall('ChatSendServerMessage', array($message));
			return;
			}
			else
			{
				$message = $aseco->formatColors('{#server}>{#emotic} Track ID not found! Type {#highlite}/list {#emotic}to see all tracks.');
				$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
				return;
			}
	}
	elseif ($command['params'] == 'list')
	{
		if (sizeof($jukebox) > 0)
		{
			$message = '{#server}> {#emotic}Current tracks in the jukebox : ';
			$i = 1;
			foreach($jukebox as $item)
			{
				$message .= '{#highlite}' . $i . '{#emotic}. [{#highlite}' . $item['Name'] . '{#emotic}], ';
				$i++;
			}
			$message = substr($message, 0, strlen($message)-2);
			$message = $aseco->formatColors($message);
			$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
			return;
		}
		else
		{
			$message = $aseco->formatColors('{#server}>{#emotic} No tracks in the jukebox, use {#highlite}/jukebox <Track_ID> {#emotic}to add one...');
			$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
			return;
		}
	}
	elseif ($command['params'] == '')
	{
		$message = $aseco->formatColors('{#server}>{#emotic} You have to include a Track ID, e.g {#highlite}/jukebox 12 - Type {#highlite}/list{#emotic} help for more info');
		$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
		return;
	}
}

function chat_y(&$aseco, &$command) {
	global $tmxadd, $tmxvotes, $jukebox;
	if (in_array($command['author']->login, $tmxvotes)) {
		$message = $aseco->formatColors('{#server}>{#emotic} You have already voted!');
		$aseco->addCall('ChatSendServerMessageToLogin', array($message, $command['author']->login));
		return;
	}
	if (sizeof($tmxadd) > 0 && $tmxadd['votes'] > 0) {
		$votereq = $tmxadd['votes'];
		$votereq--;
		if ($votereq > 0) {
			$tmxadd['votes'] = $votereq;
			$msg = '{#server}>{#highlite} ' . $votereq . '{#emotic} more votes needed. Type {#highlite}/y{#emotic} to vote.';
			$message = $aseco->formatColors($msg);
			$aseco->addCall('ChatSendServerMessage', array($message));
			$tmxvotes[] = $command['author']->login;
			return;
		} else {
			$uid = $tmxadd['uid'];
			$jukebox[$uid]['FileName'] = $tmxadd['filename'];
			$jukebox[$uid]['Name'] = stripFormatting($tmxadd['name']);
			$jukebox[$uid]['Nick'] = $tmxadd['nick'];
			$jukebox[$uid]['tmx'] = true;
			$jukebox[$uid]['uid'] = $uid;
			$jukebox[$uid]['env'] = $tmxadd['env'];
			$msg = '{#server}>{#emotic} Vote Passed! {#highlite}' . stripFormatting($tmxadd['name']) . '{#emotic} has been added to the jukebox.';
			$message = $aseco->formatColors($msg);
			$aseco->addCall('ChatSendServerMessage', array($message));
			$tmxadd = array();
			return;
		}
	} else {
		$message = $aseco->formatColors('{#server}>{#emotic} There are no TMX votes right now. Use {#highlite}/add <tmx_id> {#emotic} to start one.');
		$aseco->addCall('ChatSendServerMessageToLogin', array($message, $command['author']->login));
	}
}

function chat_add(&$aseco, &$command) {
	global $tmxadd, $tmxvote, $tmxdir, $feature_tmxadd, $jb_env;
	$player = $command['author'];			// $aseco->server->players->getPlayer($login);
	$login = $player->login;
	$tracktypes = array();
	$tracktypes['TMO'] = array(0 => 7458, 'original');
	$tracktypes['TMS'] = array(0 => 7455, 'sunrise');
	$tracktypes['TMN'] = array(0 => 7456, 'nations');
	$tracktypes['TMU'] = array(0 => 7459, 'united');

	$params = explode(' ', trim($command['params']));
	$tmxid = $params[0];

	// if only one parm, tracktype = TMU, else validate tracktype to be TMO/TMS/TMN, error out if it's not
	// parm 1 should always be numeric (tested further down)

	if ($feature_tmxadd == true) {
		if (sizeof($tmxadd) > 0) {
			$message = $aseco->formatColors('{#server}>{#emotic} There is already a TMX vote in progress. Wait for this to complete.');
			$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
			return;
		}
		if ( $tmxid == 'trackref' )
			{
			build_tmx_trackref($aseco);
			$message = $aseco->formatColors("{#server}>{#emotic} Wrote trackref.txt");
			$aseco->addcall('ChatSendServerMessageToLogin', array($message, $login));
			return;
			}
		if (is_numeric($tmxid) && $tmxid >= 0) {
			$tmxid = ltrim($tmxid, '0');
			if ( count($params) > 1 )
				{
				$tracktype = strtoupper($params[1]);
				if ( !isset($tracktypes[$tracktype]) )
					{
					$message = $aseco->formatColors("{#server}>{#emotic} '" . $tracktype . "' is not valid, try again with TMO/TMS/TMN/TMU.");
					$aseco->addcall('ChatSendServerMessageToLogin', array($message, $login));
					return;
					}
				}
			else
				{
				$tracktype = 'TMU';
				}
			$remotefile = 'http://' . $tracktypes[$tracktype][1] . '.tm-exchange.com/get.aspx?action=trackgbx&id=' . $tmxid;
			if (!$stream = fopen($remotefile, 'r')) {
				$message = $aseco->formatColors('{#server}>{#emotic} Track not found, or error downloading. Please check the TrackID.');
				$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
				return;
			} else {
				if (!$file = stream_get_contents($stream)) {
					$message = $aseco->formatColors('{#server}>{#emotic} Error Saving file - unable to get data. Please contact admin.');
					$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
					return;
				}
				$sepchar = substr($aseco->server->trackdir, -1, 1);
				$partialdir = $tmxdir . $sepchar . $tmxid . '.challenge.gbx';
				$localfile = $aseco->server->trackdir . $partialdir;
				if ( $aseco->debug )
					{
					$aseco->console_text('/add - tmxdir=' . $tmxdir);
					$aseco->console_text('/add - path + filename=' . $partialdir);
					$aseco->console_text('/add - aseco->server->trackdir = ' . $aseco->server->trackdir);
					}
				if (file_exists($localfile)) {
					if (!unlink($localfile)) {
						$message = $aseco->formatColors('{#server}>{#emotic} Error erasing old file. Please contact admin');
						$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
						return;
					}
				}
				if (!$lfile = fopen($localfile, 'w+')) {
					$message = $aseco->formatColors('{#server}>{#emotic} Error creating file. Please contact admin');
					$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
					return;
				}
				if (!fwrite($lfile, $file)) {
					$message = $aseco->formatColors('{#server}>{#emotic} Error saving file - unable to write data. Please contact admin');
					$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
					return;
				}
				fclose($lfile);		// can't do filesize while it's open, might have also caused problems with tmserver giving back info
				if (filesize($localfile) == $tracktypes[$tracktype][0]) {
					$message = $aseco->formatColors('{#server}>{#emotic} No such track on TMX');
					$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
					unlink($localfile);
					return;
				} else
					{
					$newtrk = getChallengeData($localfile, True);		// 2nd parm is whether or not to get players & votes required
					if ( $newtrk['votes'] == 500 && $newtrk['name'] == 'Not a GBX file')
						{
						$message = $aseco->formatColors('{#server}>{#emotic} No such track on TMX');
						$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
						unlink($localfile);
//						return;
						}
					else
						{
						if ( isset($jb_env) && strlen($jb_env) > 0)			// it exists and has data, go check it
							{
							if ( stripos($jb_env, $newtrk['environment']) === false )
								{
								$msg = '{#server}>{#highlite} Environment (' . $newtrk['environment'] . ') not acceptable on this server. Try again.';
								$message = $aseco->formatColors($msg);
								$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
								unlink($localfile);
								return;
								}
							}
						$ctr = 0;

						getAllChallenges($player, '*', '*');		// populate players tracklist with all current tracks
						foreach($player->tracklist as $key)
							{
							if ( $key['uid'] == $newtrk['uid'] )
								{
								$msg = '{#server}>{#highlite} File already in track list, added via jukebox';
								$message = $aseco->formatColors($msg);
								$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
								$command['params'] = $ctr;
								chat_jukebox($aseco, $command);
								return;
								}
							$ctr++;
							}
						$tmxadd['filename'] = $partialdir;
						$tmxadd['votes'] = $newtrk['votes'];
						$tmxadd['name'] = stripFormatting($newtrk['name']);
						$tmxadd['nick'] = stripFormatting($command['author']->nickname);
						$tmxadd['uid'] = $newtrk['uid'];
						$tmxadd['env'] = $newtrk['environment'];
						$tmxvote = true;
						$msg = '{#server}>{#highlite} ' . $tmxadd['nick'] . '{#emotic} is requesting {#highlite}' . $tmxadd['name'] . '$z{#emotic} from TMX. This will require {#highlite}' . $tmxadd['votes'] . '{#emotic} votes to pass. Type {#highlite}/y {#emotic} to vote.';
						$message = $aseco->formatColors($msg);
						$aseco->addCall('ChatSendServerMessage', array($message));
						}
				}
			}
		} else {
			$message = $aseco->formatColors('{#emotic}You must include a TMX track ID');
			$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
		}
	}
else
	{
	$message = $aseco->formatColors('{#emotic}/add is not currently enabled on this server');
	$aseco->addCall('ChatSendServerMessageToLogin', array($message, $login));
	}
}

function build_tmx_trackref(&$aseco)
	{
	global $tmxdir;
	$td = $aseco->server->trackdir . $tmxdir;

	if ( is_dir($td) )
		{
		$dir = opendir($td);
		$fp = fopen($td . '/trackref.txt', 'w');
		while ( ($file = readdir($dir)) !== false )
			{
			if ( substr($file, -4) == '.gbx' )
				{
				$ci = getChallengeData($td . '/' . $file, false);
				$file = str_replace('.challenge.gbx', '', $file);
				fwrite($fp, $file . "\t" . $ci['tracktype'] . "\t" . $ci['author'] . "\t" . stripCodes($ci['name']) . "\t" . $ci['coppers'] . CRLF);
				}
			}
		fclose($fp);
		closedir($dir);
		}

	}  //  build_tmx_trackref

function getChallengeInfo($filename) {
	global $aseco, $tmxvoteratio;
	$resp = $aseco->client->addCall('AddChallenge', array($filename));
	if (!$aseco->client->multiquery()) {
		trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage());
		$response = array();
		$ret['name'] = 'failed';
		$ret['votes'] = 500;
		return $ret;
	} else {
		$track = $aseco->client->addCall('GetChallengeInfo', array($filename));
		if (!$aseco->client->multiquery()) {
			trigger_error('[' . $aseco->client->getErrorCode() . '] ' . $aseco->client->getErrorMessage());
			$response = array();
			$ret['name'] = 'failed';
			$ret['votes'] = 500;
			return $ret;
		} else {
			$response = $aseco->client->getResponse();
			$ret['name'] = $response[$track][0]['Name'];
			$nbplrs = sizeof($aseco->server->players->player_list);
			$ret['votes'] = floor($nbplrs * $tmxvoteratio);
			if ($ret['votes'] < 1) {
				$ret['votes']++;
			}
		}
	}
}

function chat_list(&$aseco, &$command)
	{
	$player = $command['author'];
	$login = $player->login;

	// split params into array
	if ( strlen(trim($command['params'])) > 0 )
		{
		$params = explode(' ', trim($command['params']));
		}
	else
		{
		$params = array();
		}

	$cmdcount = count($params);
	$env = "*";

	if ( $cmdcount > 0 )
		{
		foreach($params as $parm)
			{
			if ( substr($parm, 0, 4) == 'env:' )
				{
				$command['params'] = trim(str_replace($parm, '', $command['params']));	// if env: was here, remove it so you can search for stuff later
				$env = substr($parm, 4);			// get the text of the environment
				$cmdcount--;
				}
			}
		}

	if ( $cmdcount == 1 && $params[0] == 'help' )
		{
		$lf = "\n";
		$help = '/list will show tracks in rotation on the server' . $lf;
		$help .= '  - nofinish, tracks you haven\'t completed' . $lf;
		$help .= '  - xxx, where xxx is part of a track or author name' . $lf;
		$help .= '  - env:bay/coast/desert/island/rally/snow/stadium to list all tracks of that type' . $lf;
		$help .= '  - karma +/-#, shows all with karma <= or >= given value' . $lf;
		$help .= '    (example: /list karma -3 will show all tracks with karma <= -3)';

		popup_msg($login, $help);
		return;
		}
	elseif ( $cmdcount == 1 && ($params[0] == "nofinish") )
		{
		getChallengesNoFinish($player);
		}
	elseif ( $cmdcount == 2 && ($params[0] == "karma" ) )
		{
		$karmaval = intval($params[1]);
		getChallengesByKarma($player, $karmaval);
		}
	elseif ( $cmdcount == 0 )
		{
		$wildcard = "*";
		getAllChallenges($player, $wildcard, $env);
		}
	elseif ( $cmdcount >=1 && strlen($params[0]) > 0 )
		{
		$wildcard = $command['params'];
		getAllChallenges($player, $wildcard, $env);
		}

	if ( sizeof($player->tracklist) == 0 )
		{
		$aseco->addCall('ChatSendServerMessageToLogin', array("No tracks found, try again.", $login));
		return;
		}

	show_multi_msg($player);
	}

function event_jukebox_message(&$aseco, &$answer) {
	$player = $answer['author'];
	if ( count($player->tracklist) == 0 || $answer[2] < 1001 || empty($player->msgs))
		{
		return;
		}
	elseif(isset($player->tracklist[$answer[2] - 1001]))		// tracks are zero based, not 1
		{
		$answer['params'] = '' . $answer[2] - 1000;             // arg for jukebox however, is 1 based
		$player->msgs = array();
		chat_jukebox($aseco, $answer);
		}
	}  //  event_jukebox_message

Aseco::registerEvent('onPlayerServerMessageAnswer', 'event_jukebox_message');		// RASP .4.1
?>
